#include <vector>
#include <stdlib.h>
#include <Windows.h>
#include "bccfe.h"

using namespace win32fe;

bcc::bcc() {
  compileoutflag = "-o";
  linkoutflag    = "-e";

  OutputFlag = compilearg.end();
}

int bcc::Parse(void) {
  if (!compiler::Parse()) {
    if (!verbose) {
      compilearg.push_back("-q");
    }
  } else {
    return(-1);
  }
  return(0);
}

int bcc::Link(void) {
  if (OutputFlag==compilearg.end()) {
    std::string tempstr = linkoutflag + file.front();
    tempstr.replace(tempstr.rfind("."),std::string::npos,".exe");
    linkarg.push_front(tempstr);
  }

  /* Copy all <file>.o in file list to %TMP%\<file>.obj */ 
  int i,ierr=0;
  LI f;
  char path[MAX_PATH]; /* Windows.h */
  GetTempPath(MAX_PATH,path); /* Windows.h */
  /* Change these to list<string> someday */
  std::vector<std::string> ext(file.size(),""); /* File extensions */
  std::vector<std::string> temp(file.size(),"");

  for (i=0,f=file.begin();f!=file.end();i++,f++) {
    int n = (*f).rfind(".");
    ext[i] = (*f).substr(n,std::string::npos);
    if (ext[i] == ".o") {
      temp[i]=*f;
      std::string outfile = (std::string)path + (*f).substr(0,n) + ".obj";
      std::string copy = "copy " + temp[i] + " " + outfile;
      if (verbose) {
        std::cout << copy << std::endl;
      }
      CopyFile(temp[i].c_str(),outfile.c_str(),FALSE); /* Windows.h */
      /* Replace <file>.o in the file list with %TMP%\<file>.obj */
      f = file.erase(f);
      f = file.insert(f,outfile);
    }
  }

  /* Link, note linkargs before files */
  f = compilearg.begin();
  std::string link = *f++;
  Merge(link,compilearg,f);
  f = linkarg.begin();
  Merge(link,linkarg,f);
  f = file.begin();
  Merge(link,file,f);
  if (verbose) {
    std::cout << link << std::endl;
  }
  ierr = system(link.c_str());

  /* Remove %TMP%\file.obj's */
  for (i=0,f=file.begin();f!=file.end();i++,f++) {
    if (ext[i] == ".o") {
      std::string del = "del " + *f;
      if (verbose) {
        std::cout << del << std::endl;
      }
      DeleteFile((*f).c_str()); /* Windows.h */
      f = file.erase(f);
      f = file.insert(f,temp[i]);
    }
  }
  return(ierr);
}

void bcc::Help(void) {
  compiler::Help();
  std::cout << "bcc32 specific help:" << std::endl;
  std::cout << "  Note: The bcc32 option -l conflicts with the win32fe use of -l." << std::endl;
  std::cout << "        The following additional options are enabled for bcc32." << std::endl << std::endl;
  std::cout << "  -l:<flag>    enables <flag> for the linker, ilink32.exe" << std::endl;
  std::cout << "  -l:-<flag>   disables <flag> for the linker, ilink32.exe" << std::endl << std::endl;
  std::cout << "=========================================================================" << std::endl << std::endl;
  std::string help = compilearg.front();
  system(help.c_str());
}

void bcc::FindInstallation(void) {
  bool oldwoff = woff;
  woff = true;
  tool::FindInstallation();
  woff = oldwoff;
  if (!GetShortPath(InstallDir)) {
    std::string trypath[3];
    trypath[0]="C:\\Borland\\BCC55\\Bin\\bcc32.exe";
    trypath[1]="C:\\Program Files\\Borland\\BCC55\\Bin\\bcc32.exe";
    trypath[2]="/software/Borland/BCC55/Bin/bcc32.exe";
    for (int i=0;i<3;i++) {
      if (!GetShortPath(trypath[i])) {
        InstallDir = "";
      } else {
        InstallDir = trypath[i].substr(0,trypath[i].rfind("\\"));
        InstallDir = InstallDir.substr(0,InstallDir.rfind("\\")+1);
        break;
      }
    }
  }
  if (InstallDir!="") {
    LI FoundPathHere=arg.end();
    arg.push_back("--path");
    FoundPathHere--;
    arg.push_back(InstallDir+"Bin");
    tool::FoundPath(FoundPathHere);
  } else {
    std::cerr << "Error: bcc32.exe installation not found." << std::endl;
    std::cerr << "  Specify the complete path to bcc32.exe with --use or --path" << std::endl;
    std::cerr << "  Use --help for more information on win32fe options." << std::endl << std::endl;
  }
}

void bcc::Foundl(LI &i) {
  std::string temp = *i;
  if (temp[2]==':') {
    linkarg.push_back("-l" + temp.substr(3));
  } else {
    compiler::Foundl(i);
  }
} 
