#include "libfe.h"

using namespace win32fe;
  
int lib::Archive(void) {
  int ierr=0;
  LI li = archivearg.begin();
  std::string header = *li++;
  std::string archivename = archivearg.back();
  archivearg.pop_back();
  if (GetShortPath(archivename)) {
    /* If archive exists, concatenate archive with list */
    file.push_front(archivename);
  } else {
    /* If archive does not exist, build short form of new name (without file part) */
    std::string::size_type loc=archivename.rfind("\\");
    if (loc==std::string::npos) {
      loc = archivename.rfind("/"); 
    }
    if (loc!=std::string::npos) {
      std::string archivedir=archivename.substr(0,loc);
      if (GetShortPath(archivedir)) {
        archivename = archivedir + archivename.substr(loc);
      }
    }
  }
  archivearg.push_back("-out:" + archivename);
  Merge(header,archivearg,li);
//    PrintListString(archivearg);
  li = file.begin();
  while ((li != file.end()) && (!ierr)) {
    std::string archive = header;
    Merge(archive,file,li);
    if (verbose) {
      std::cout << archive << std::endl;
    }
    ierr = system(archive.c_str());
    /* On the first pass, archivearg.back == "-out:archivename" */
    /* Otherwise, archivearg.back == "archivename" */ 
    /* and "-out:..." is the arg before the back */
    if (archivearg.back()!=archivename) { 
      archivearg.push_back(archivename);
      Merge(header,archivearg,--archivearg.end());
    }
  }
  return(ierr);
}

void lib::Help(void) {
  archiver::Help();
  std::cout << "lib specific help:" << std::endl;
  std::cout << "  Note: win32fe will check to see if the library exists, and will modify" << std::endl;
  std::cout << "        the arguments to lib accordingly.  As a result, specifying" << std::endl;
  std::cout << "        -out:<libname> is unnecessary and may lead to unexpected results." << std::endl << std::endl;
  std::cout << "=========================================================================" << std::endl << std::endl;
  
  std::string help = archivearg.front();
  help += " -? 2>&1"; 
  system(help.c_str());
}

int lib::Parse(void) {
  if (!archiver::Parse()) {
    if (!verbose) {
      archivearg.push_back("-nologo");
    }
  } else {
    return(-1);
  }
  return(0);
}

void lib::FindInstallation(void) {
  tool::FindInstallation();
  std::string::size_type n = InstallDir.length()-1;
  VisualStudioDir = InstallDir.substr(0,n);
  n = VisualStudioDir.rfind("\\");
  VisualStudioDir = VisualStudioDir.substr(0,n+1);
  VSVersion = InstallDir.substr(0,InstallDir.length()-1);
  VSVersion = VSVersion.substr(VisualStudioDir.length());
}

void lib::AddPaths(void) {
  /* Find required .dll's */
  std::string addpath;
  /* This is ugly and perhaps each version should have their own class */
  bool KnownVersion=false;
  if (VSVersion=="VC98") {
    addpath = VisualStudioDir + "Common\\MSDev98\\Bin";
    KnownVersion=true;
  } else if (VSVersion=="VC7") {
    addpath = VisualStudioDir + "Common7\\IDE";
    KnownVersion=true;
  } else {
    if (!woff) {
      std::cerr << "Warning: win32fe Visual Studio version not recognized." << std::endl;
    }
  }
  if (KnownVersion) {
    arg.push_back("--path");
    LI i = arg.end();
    i--;
    GetShortPath(addpath);
    arg.push_back(addpath);
    FoundPath(i);
  }
}  
